aboutsummaryrefslogtreecommitdiff
path: root/hw/i386
diff options
context:
space:
mode:
authorZhao Liu <zhao1.liu@intel.com>2024-04-24 23:49:23 +0800
committerPaolo Bonzini <pbonzini@redhat.com>2024-05-22 19:43:29 +0200
commitb17a26bc4b0289ca7415f821bb0a89a8a333cea5 (patch)
tree03f0cf09237244fb115766dbac28d1042e3eb28b /hw/i386
parent5304873acd12f7a4ddb07d3560ac5ac02a0e1060 (diff)
downloadqemu-b17a26bc4b0289ca7415f821bb0a89a8a333cea5.zip
qemu-b17a26bc4b0289ca7415f821bb0a89a8a333cea5.tar.gz
qemu-b17a26bc4b0289ca7415f821bb0a89a8a333cea5.tar.bz2
i386: Support module_id in X86CPUTopoIDs
Add module_id member in X86CPUTopoIDs. module_id can be parsed from APIC ID, so also update APIC ID parsing rule to support module level. With this support, the conversions with module level between X86CPUTopoIDs, X86CPUTopoInfo and APIC ID are completed. module_id can be also generated from cpu topology, and before i386 supports "modules" in smp, the default "modules per die" (modules * clusters) is only 1, thus the module_id generated in this way is 0, so that it will not conflict with the module_id generated by APIC ID. Tested-by: Yongwei Ma <yongwei.ma@intel.com> Signed-off-by: Zhuocheng Ding <zhuocheng.ding@intel.com> Co-developed-by: Zhuocheng Ding <zhuocheng.ding@intel.com> Signed-off-by: Zhao Liu <zhao1.liu@intel.com> Tested-by: Babu Moger <babu.moger@amd.com> Message-ID: <20240424154929.1487382-16-zhao1.liu@intel.com> Signed-off-by: Paolo Bonzini <pbonzini@redhat.com>
Diffstat (limited to 'hw/i386')
-rw-r--r--hw/i386/x86-common.c27
-rw-r--r--hw/i386/x86.c4
2 files changed, 21 insertions, 10 deletions
diff --git a/hw/i386/x86.c b/hw/i386/x86.c
index d5d668e..a4aa8e0 100644
--- a/hw/i386/x86.c
+++ b/hw/i386/x86.c
@@ -135,6 +135,10 @@ static const CPUArchIdList *x86_possible_cpu_arch_ids(MachineState *ms)
ms->possible_cpus->cpus[i].props.has_die_id = true;
ms->possible_cpus->cpus[i].props.die_id = topo_ids.die_id;
}
+ if (ms->smp.modules > 1) {
+ ms->possible_cpus->cpus[i].props.has_module_id = true;
+ ms->possible_cpus->cpus[i].props.module_id = topo_ids.module_id;
+ }
ms->possible_cpus->cpus[i].props.has_core_id = true;
ms->possible_cpus->cpus[i].props.core_id = topo_ids.core_id;
ms->possible_cpus->cpus[i].props.has_thread_id = true;
diff --git a/hw/i386/x86-common.c b/hw/i386/x86-common.c
index 34dfa89..4d27279 100644
--- a/hw/i386/x86-common.c
+++ b/hw/i386/x86-common.c
@@ -283,12 +283,9 @@ void x86_cpu_pre_plug(HotplugHandler *hotplug_dev,
/*
* If APIC ID is not set,
* set it based on socket/die/core/thread properties.
*/
if (cpu->apic_id == UNASSIGNED_APIC_ID) {
- int max_socket = (ms->smp.max_cpus - 1) /
- smp_threads / smp_cores / ms->smp.dies;
-
/*
* die-id was optional in QEMU 4.0 and older, so keep it optional
* if there's only one die per socket.
@@ -300,9 +297,9 @@ void x86_cpu_pre_plug(HotplugHandler *hotplug_dev,
if (cpu->socket_id < 0) {
error_setg(errp, "CPU socket-id is not set");
return;
- } else if (cpu->socket_id > max_socket) {
+ } else if (cpu->socket_id > ms->smp.sockets - 1) {
error_setg(errp, "Invalid CPU socket-id: %u must be in range 0:%u",
- cpu->socket_id, max_socket);
+ cpu->socket_id, ms->smp.sockets - 1);
return;
}
if (cpu->die_id < 0) {
@@ -334,17 +331,27 @@ void x86_cpu_pre_plug(HotplugHandler *hotplug_dev,
topo_ids.die_id = cpu->die_id;
topo_ids.core_id = cpu->core_id;
topo_ids.smt_id = cpu->thread_id;
+
+ /*
+ * TODO: This is the temporary initialization for topo_ids.module_id to
+ * avoid "maybe-uninitialized" compilation errors. Will remove when
+ * X86CPU supports module_id.
+ */
+ topo_ids.module_id = 0;
+
cpu->apic_id = x86_apicid_from_topo_ids(&topo_info, &topo_ids);
}
cpu_slot = x86_find_cpu_slot(MACHINE(x86ms), cpu->apic_id, &idx);
if (!cpu_slot) {
x86_topo_ids_from_apicid(cpu->apic_id, &topo_info, &topo_ids);
+
error_setg(errp,
- "Invalid CPU [socket: %u, die: %u, core: %u, thread: %u] with"
- " APIC ID %" PRIu32 ", valid index range 0:%d",
- topo_ids.pkg_id, topo_ids.die_id, topo_ids.core_id, topo_ids.smt_id,
- cpu->apic_id, ms->possible_cpus->len - 1);
+ "Invalid CPU [socket: %u, die: %u, module: %u, core: %u, thread: %u]"
+ " with APIC ID %" PRIu32 ", valid index range 0:%d",
+ topo_ids.pkg_id, topo_ids.die_id, topo_ids.module_id,
+ topo_ids.core_id, topo_ids.smt_id, cpu->apic_id,
+ ms->possible_cpus->len - 1);
return;
}
@@ -283,12 +283,9 @@ void x86_cpu_pre_plug(HotplugHandler *hotplug_dev,
/*
* If APIC ID is not set,
- * set it based on socket/die/core/thread properties.
+ * set it based on socket/die/module/core/thread properties.
*/
if (cpu->apic_id == UNASSIGNED_APIC_ID) {